@@ -1,47 +1,19 @@
-import traceback
-
 import cgi
 import inspect
 import io
 import json
+import traceback
 import urllib
+from functools import update_wrapper, partial
+
 from aws_xray_sdk.core import xray_recorder
 from botocore.exceptions import BotoCoreError
 from chalice import AuthResponse, Response
-from functools import update_wrapper, partial
 from requests_toolbelt.multipart import MultipartDecoder
 
-from coworks import aws
-from coworks.error import CwsError
-from .utils import HTTP_METHODS, class_auth_methods, class_cws_methods, make_absolute, path_join
-
-
-class EntryPoint:
-    """An entry point is an API entry defined on a component, with a specific authorization function and
-    is response function."""
-
-    def __init__(self, component, auth, fun):
-        self.component = component
-        self.auth = auth
-        self.fun = fun
-
-
-class Entry(dict):
-
-    @property
-    def authorizer(self):
-        for method in HTTP_METHODS:
-            m = self.get(method.upper())
-            if m:
-                return m.auth
-
-    def call_get(self, *args, **kwargs):
-        method = self["GET"]
-        return method.fun(method.component, *args, **kwargs)
-
-    def call_post(self, *args, **kwargs):
-        method = self["POST"]
-        return method.fun(method.component, *args, **kwargs)
+from .aws import AwsS3Session
+from .error import CwsError
+from .utils import class_auth_methods, class_cws_methods, make_absolute, path_join
 
 
 class CoworksMixin:
@@ -66,12 +38,12 @@ class CoworksMixin:
         # To register a function, use the :meth:`handle_exception` decorator.
         self.handle_exception_funcs = []
 
-        self.aws_s3_sfn_data_session = aws.AwsS3Session(env_var_access_key="AWS_RUN_ACCESS_KEY_ID",
-                                                        env_var_secret_key="AWS_RUN_SECRET_KEY",
-                                                        env_var_region="AWS_RUN_REGION")
-        self.aws_s3_form_data_session = aws.AwsS3Session(env_var_access_key="AWS_FORM_DATA_ACCESS_KEY_ID",
-                                                         env_var_secret_key="AWS_FORM_DATA_SECRET_KEY",
-                                                         env_var_region="AWS_FORM_DATA_REGION")
+        self.aws_s3_sfn_data_session = AwsS3Session(env_var_access_key="AWS_RUN_ACCESS_KEY_ID",
+                                                    env_var_secret_key="AWS_RUN_SECRET_KEY",
+                                                    env_var_region="AWS_RUN_REGION")
+        self.aws_s3_form_data_session = AwsS3Session(env_var_access_key="AWS_FORM_DATA_ACCESS_KEY_ID",
+                                                     env_var_secret_key="AWS_FORM_DATA_SECRET_KEY",
+                                                     env_var_region="AWS_FORM_DATA_REGION")
 
     def before_first_activation(self, f):
         """Registers a function to be run before the first activation of the microservice.
@@ -126,7 +98,7 @@ class CoworksMixin:
 
         # Global authorization function may be redefined
         auth_fun = app.config.auth if app.config.auth else class_auth_methods(app)
-        auth = self._create_auth_proxy(auth_fun) if auth_fun else None
+        auth = self._create_auth_proxy(app, auth_fun) if auth_fun else None
 
         # Adds entrypoints
         methods = class_cws_methods(self)
@@ -158,26 +130,30 @@ class CoworksMixin:
 
             # Creates the entry
             if hide_routes is False or (type(hide_routes) is list and entry_path not in hide_routes):
+                if app.entries is None:
+                    app.entries = {}
+                if entry_path not in app.entries:
+                    app.entries[entry_path] = {}
                 if method in app.entries[entry_path]:
                     raise CwsError(f"The method {method} is already defined for the route {make_absolute(entry_path)}")
-                app.entries[entry_path][method] = EntryPoint(self, auth, fun)
+                app.add_entry(entry_path, method, auth, fun)
                 app.route(f"{make_absolute(route)}", methods=[method], authorizer=auth, cors=app.config.cors,
                           content_types=list(app.config.content_type))(proxy)
 
-    def _create_auth_proxy(self, auth_method):
+    def _create_auth_proxy(self, app, auth_method):
 
         def proxy(auth_activation):
             subsegment = xray_recorder.current_subsegment()
             try:
-                auth = auth_method(self, auth_activation)
+                auth = auth_method(auth_activation)
                 if subsegment:
                     subsegment.put_metadata('result', auth)
             except Exception as e:
-                self.log.info(f"Exception : {str(e)}")
+                app.log.info(f"Exception : {str(e)}")
                 traceback.print_exc()
                 if subsegment:
                     subsegment.add_exception(e, traceback.extract_stack())
-                return Response(body=str(e), status_code=500)
+                return AuthResponse(routes=[], principal_id='user')
 
             if type(auth) is bool:
                 if auth:
