@@ -5,7 +5,7 @@ from chalice import BadRequestError, NotFoundError
 from ..coworks import TechMicroService, aws
 
 
-class S3MicroService(aws.Boto3Mixin, TechMicroService):
+class S3MicroService(TechMicroService):
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
